@@ -5,21 +5,19 @@ package HTML::AsSubs;
 
 use warnings;
 use strict;
-use vars qw(@ISA @EXPORT);
 
-our $VERSION = '4.903'; # TRIAL VERSION from OurPkgVersion
+our $VERSION = '5.900'; # TRIAL VERSION from OurPkgVersion
 
 require HTML::Element;
 require Exporter;
-@ISA = qw(Exporter);
+our @ISA = qw(Exporter);
 
 # Problem: exports so damned much.  Has no concept of "export only HTML4
 #  elements".  TODO:?? make something that make functions that just
 #  wrap XML::Generator calls?
 
 
-use vars qw(@TAGS);
-@TAGS = qw(html
+our @TAGS = qw(html
     head title base link meta isindex nextid script style
     body h1 h2 h3 h4 h5 h6 p pre div blockquote
     a img br hr
@@ -36,6 +34,8 @@ use vars qw(@TAGS);
     frame frameset noframe
 );
 
+our @EXPORT;
+
 for (@TAGS) {
     my $code;
     $code = "sub $_ { _elem('$_', \@_); }\n";
@@ -63,6 +63,7 @@ sub _elem {
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
@@ -72,9 +73,9 @@ HTML::AsSubs - functions that construct a HTML syntax tree
 =head1 VERSION
 
 B<This is a development release for testing purposes only.>
-This document describes version 4.903 of
-HTML::AsSubs, released June 8, 2012
-as part of HTML-Tree.
+This document describes version 5.900 of
+HTML::AsSubs, released December 15, 2012
+as part of L<HTML-Tree|HTML::Tree>.
 
 =head1 SYNOPSIS
 
@@ -201,4 +202,3 @@ will be useful, but without any warranty; without even the implied
 warranty of merchantability or fitness for a particular purpose.
 
 =cut
-
