@@ -8,13 +8,13 @@ package HTML::Element::traverse;
 use warnings;
 use strict;
 
-our $VERSION = '4.903'; # TRIAL VERSION from OurPkgVersion
+our $VERSION = '5.900'; # TRIAL VERSION from OurPkgVersion
 
 use HTML::Element ();
 1;
 
-
 __END__
+
 =pod
 
 =head1 NAME
@@ -24,9 +24,9 @@ HTML::Element::traverse - discussion of HTML::Element's traverse method
 =head1 VERSION
 
 B<This is a development release for testing purposes only.>
-This document describes version 4.903 of
-HTML::Element::traverse, released June 8, 2012
-as part of HTML-Tree.
+This document describes version 5.900 of
+HTML::Element::traverse, released December 15, 2012
+as part of L<HTML-Tree|HTML::Tree>.
 
 =head1 SYNOPSIS
 
@@ -373,4 +373,3 @@ L<< http://github.com/madsen/HTML-Tree >>.
 Copyright 2000,2001 Sean M. Burke
 
 =cut
-
