@@ -7,7 +7,7 @@ package HTML::Tree;
 use warnings;
 use strict;
 
-our $VERSION = '4.903'; # TRIAL VERSION from OurPkgVersion
+our $VERSION = '5.900'; # TRIAL VERSION from OurPkgVersion
 
 use HTML::TreeBuilder ();
 
@@ -30,17 +30,28 @@ sub new_from_content {
     goto &HTML::TreeBuilder::new_from_content;
 }
 
+sub new_from_string {
+    shift;
+    unshift @_, 'HTML::TreeBuilder';
+    goto &HTML::TreeBuilder::new_from_string;
+}
+
 sub new_from_url {
     shift;
     unshift @_, 'HTML::TreeBuilder';
     goto &HTML::TreeBuilder::new_from_url;
 }
 
+sub new_from_http {
+    shift;
+    unshift @_, 'HTML::TreeBuilder';
+    goto &HTML::TreeBuilder::new_from_http;
+}
+
 1;
 
-
-
 __END__
+
 =pod
 
 =head1 NAME
@@ -50,9 +61,9 @@ HTML::Tree - build and scan parse-trees of HTML
 =head1 VERSION
 
 B<This is a development release for testing purposes only.>
-This document describes version 4.903 of
-HTML::Tree, released June 8, 2012
-as part of HTML-Tree.
+This document describes version 5.900 of
+HTML::Tree, released December 15, 2012
+as part of L<HTML-Tree|HTML::Tree>.
 
 =head1 SYNOPSIS
 
@@ -128,10 +139,18 @@ Redirects to L<HTML::TreeBuilder/new_from_file>.
 
 Redirects to L<HTML::TreeBuilder/new_from_content>.
 
+=head2 new_from_string
+
+Redirects to L<HTML::TreeBuilder/new_from_string>.
+
 =head2 new_from_url
 
 Redirects to L<HTML::TreeBuilder/new_from_url>.
 
+=head2 new_from_http
+
+Redirects to L<HTML::TreeBuilder/new_from_http>.
+
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
@@ -158,6 +177,13 @@ L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=HTML-Tree>
 
 L<http://search.cpan.org/dist/HTML-Tree>
 
+=item * Stack Overflow
+
+L<http://stackoverflow.com/questions/tagged/html-tree>
+
+If you have a question about how to use HTML-Tree, Stack Overflow is
+the place to ask it.  Make sure you tag it both C<perl> and C<html-tree>.
+
 =back
 
 =head1 SEE ALSO
@@ -249,4 +275,3 @@ will be useful, but without any warranty; without even the implied
 warranty of merchantability or fitness for a particular purpose.
 
 =cut
-
